/*
$Revision: 1.10 $
$Date: 2009/11/20 00:29:16 $
$Author: apollan $
$Name:  $
$Id: cbceval.c,v 1.10 2009/11/20 00:29:16 apollan Exp $
*/

/*-------------------------------------------------------------------*/
/* CBCEVAL.C   Antti Pollanen                                        */
/*                                                                   */
/* Console interface for categorical clustering evaluation module.   */
/*                                                                   */
/*                                                                   */
/* Naming conventions used in the code                               */
/*                                                                   */
/*    TS        training set (data objects)                          */
/*    CB        codebook (cluster representatives, centroids)        */
/*    CCB       categorical data codebook                            */
/*    P         partitioning (pointing from TS to CB)                */
/*                                                                   */
/*    p-prefix  pointer, e.g. pTS is pointer to the training set TS  */
/*                                                                   */
/*-------------------------------------------------------------------*/

#define  ProgName       "cbceval"
#define  VersionNumber  "0.33"
#define  LastUpdated    "15.10.2009" /* AP */
#define  MAXFILENAME    64
#define  FACTFILE       "cbceval.fac"

/*-------------------------------------------------------------------*/

#include "cb.h"
#include "file.h"
#include "interfc.h"
#include "memctrl.h"

#include "ctk.h"
#include "ceval.h"

#include "parametr.c"


/* ========================== PROTOTYPES =========================== */

void PrintInfo(void);

int main(int argc, char* argv[]);


/* ========================== FUNCTIONS ============================ */

void PrintInfo(void)
  {
  PrintMessage("\n%s   %s   %s\n", ProgName, VersionNumber, LastUpdated);
  PrintMessage("Console interface for the categorical evaluation.\n");
  PrintMessage("Usage: %s [%coption] ", ProgName, OPTION_SYMBOL);
  PrintMessage("<result partitioning> <training set>\n");
  PrintMessage("For example: %s bridge.pa bridge.ts\n", ProgName);
  PrintMessage("\n  Options:\n");
  PrintOptions();
  PrintMessage("\n");
  }  /* PrintInfo() */

/*-------------------------------------------------------------------*/

int main(int argc, char* argv[])
  {
  TRAININGSET   TS;
  CODEBOOK      CB;
  CATCODEBOOK   CCB;
  PARTITIONING  P, P2;
  char          TSName[MAXFILENAME] = {'\0'};
  char          PAName[MAXFILENAME] = {'\0'};
  ParameterInfo paraminfo[2] = { { PAName,  FormatNamePA, 0, INFILE },
                                 { TSName,  FormatNameTS, 0, INFILE }  };
  double error = 0.0;
  int outlierCluster;
  PARTITIONING* pP = &P;

  ParseParameters(argc, argv, 2, paraminfo);

  /* result codebook exists and we are told not to overwrite */
  if (!ExistFile(PAName))
    {
    ErrorMessage("\nERROR: File doesn't exist: %s\n\n", PAName);
    ExitProcessing(FATAL_ERROR);
    }

  /* training set doesn't exist */
  if (!ExistFile(TSName))
    {
    ErrorMessage("\nERROR: File doesn't exist: %s\n\n", TSName);
    ExitProcessing(FATAL_ERROR);
    }

  ReadTrainingSet(TSName, &TS);
  ReadPartitioning(PAName, &P, &TS);

  outlierCluster = Value(OutlierCluster) - 1;
  if (outlierCluster >= 0)
    {
    RemoveOutliers(&TS, &P, &P2, outlierCluster);
    pP = &P2;
    FreePartitioning(&P);
    }

  CreateNewCodebook(&CB, PartitionCount(pP), &TS);
  CATCreateNewCodebook(&CCB, &CB, &TS);

  error = CATError(Value(EvaluationMethod), &CCB, pP, &TS, Value(QuietLevel));
  PrintMessage("%f\n", error);

  FreeCodebook(&TS);
  FreeCodebook(&CB);
  FreePartitioning(pP);
  CATFreeCodebook(&CCB);

  return EVERYTHING_OK;
  }  /* main() */
