/*
$Revision: 1.3 $
$Date: 2009/12/03 13:46:01 $
$Author: apollan $
$Name:  $
$Id: catmap.c,v 1.3 2009/12/03 13:46:01 apollan Exp $
*/

/*-------------------------------------------------------------------*/
/* CATMAP.C     Antti Pollanen                                       */
/*                                                                   */
/* Console interface for mapping.                                    */
/*                                                                   */
/*                                                                   */
/* Naming conventions used in the code                               */
/*                                                                   */
/*    TS        training set (data objects)                          */
/*    CB        codebook (cluster representatives, centroids)        */
/*    CCB       categorical data codebook                            */
/*    P         partitioning (pointing from TS to CB)                */
/*                                                                   */
/*    p-prefix  pointer, e.g. pTS is pointer to the training set TS  */
/*                                                                   */
/*-------------------------------------------------------------------*/

#define  ProgName       "catmap"
#define  VersionNumber  "Version 0.11"
#define  LastUpdated    "03.12.2009" /* AP */
#define  MAXFILENAME    64
#define  FACTFILE       "catmap.fac"

/* ----------------------------------------------------------------- */
/*      Changelog:                                                   */
/*                                                                   */
/* AP   0.11  Better userinterface                                   */
/* AP   0.10  Initial implementation                                 */
/*                                                                   */
/* ----------------------------------------------------------------- */

#include "cb.h"
#include "file.h"
#include "interfc.h"
#include "memctrl.h"
#include "random.h"

#include "ctk.h"

#include <stdlib.h>    // for srand and rand
#include <time.h>      // for time
#include "math.h"
#include "parametr.c"


/* ========================== PROTOTYPES =========================== */

void PrintInitialData(int quietLevel, char *TSInput, char *TSOutput,
  int sampleSize);

void PrintInfo(void);

int main(int argc, char* argv[]);


/* ========================== FUNCTIONS ============================ */

void PrintInfo(void)
  {
  PrintMessage("\n%s   %s   %s\n", ProgName, VersionNumber, LastUpdated);
  PrintMessage("Console interface for mapping.\n");
  PrintMessage("Usage: %s [%coption] ", ProgName, OPTION_SYMBOL);
  PrintMessage("<sample training set> <sample partitioning> <training set> <partitioning>\n");
  PrintMessage("For example: %s sample.ts sample.pa dataset.ts output.pa\n", ProgName);
  PrintMessage("\n  Options:\n");
  PrintOptions();
  PrintMessage("\n");
  }  /* PrintInfo() */

/*-------------------------------------------------------------------*/

int main(int argc, char* argv[])
  {
  TRAININGSET   SampleTS, RealTS;
  PARTITIONING  SamplePA, RealPA;
  CATCODEBOOK   CCB;
  CODEBOOK      CB;
  char          SampleTSName[MAXFILENAME] = {'\0'};
  char          SamplePAName[MAXFILENAME] = {'\0'};
  char          RealTSName  [MAXFILENAME] = {'\0'};
  char          RealPAName  [MAXFILENAME] = {'\0'};
  ParameterInfo paraminfo[4] = { { SampleTSName,  FormatNameTS, 0, INFILE },
                                 { SamplePAName,  FormatNamePA, 0, INFILE },
                                 { RealTSName,    FormatNameTS, 0, INFILE },
                                 { RealPAName,    FormatNamePA, 0, OUTFILE } };

  ParseParameters(argc, argv, 4, paraminfo);

  if (!ExistFile(SampleTSName))
    {
    ErrorMessage("\nERROR: Cannot find the file: %s\n\n", SampleTSName);
    ExitProcessing(FATAL_ERROR);
    }

  if (!ExistFile(SamplePAName))
    {
    ErrorMessage("\nERROR: Cannot find the file: %s\n\n", SamplePAName);
    ExitProcessing(FATAL_ERROR);
    }

  if (!ExistFile(RealTSName))
    {
    ErrorMessage("\nERROR: Cannot find the file: %s\n\n", RealTSName);
    ExitProcessing(FATAL_ERROR);
    }

  /* result codebook exists and we are told not to overwrite */
  if (ExistFile(RealPAName) && !Value(OverWrite))
    {
    ErrorMessage("\nERROR: File already exists: %s\n\n", RealPAName);
    ExitProcessing(FATAL_ERROR);
    }

  ReadTrainingSet(SampleTSName, &SampleTS);
  ReadPartitioning(SamplePAName, &SamplePA, &SampleTS);

  ReadTrainingSet(RealTSName, &RealTS);
  CreateNewPartitioning(&RealPA, &RealTS, PartitionCount(&SamplePA));

  CreateNewCodebook(&CB, PartitionCount(&SamplePA), &SampleTS);
  CATCreateNewCodebook(&CCB, &CB, &SampleTS);

  CATGenerateOptimalCodebook(&SamplePA, &SampleTS, &CCB, Value(RepresentativeType));
  CATGenerateOptimalPartition(&CCB, &RealTS, &RealPA, Value(MappingCriteria));

  WritePartitioning(RealPAName, &RealPA, &RealTS, Value(OverWrite));

  FreeCodebook(&SampleTS);
  FreePartitioning(&SamplePA);
  FreeCodebook(&RealTS);
  FreePartitioning(&RealPA);

  checkmemory();

  return 0;
  } /* main */

