/*
$Revision: 1.3 $
$Date: 2009/12/03 13:46:01 $
$Author: apollan $
$Name:  $
$Id: catsample.c,v 1.3 2009/12/03 13:46:01 apollan Exp $
*/

/*-------------------------------------------------------------------*/
/* CATSAMPLE.C   Antti Pollanen                                      */
/*                                                                   */
/* Console interface for sampling.                                   */
/*                                                                   */
/*                                                                   */
/* Naming conventions used in the code                               */
/*                                                                   */
/*    TS        training set (data objects)                          */
/*    CB        codebook (cluster representatives, centroids)        */
/*    CCB       categorical data codebook                            */
/*    P         partitioning (pointing from TS to CB)                */
/*                                                                   */
/*    p-prefix  pointer, e.g. pTS is pointer to the training set TS  */
/*                                                                   */
/*-------------------------------------------------------------------*/

#define  ProgName       "catsample"
#define  VersionNumber  "Version 0.11"
#define  LastUpdated    "03.12.2009" /* AP */
#define  MAXFILENAME    64
#define  FACTFILE       "catsample.fac"

/* ----------------------------------------------------------------- */
/*      Changelog:                                                   */
/*                                                                   */
/* AP   0.11  Better userinterface                                   */
/* AP   0.10  Initial implementation                                 */
/*                                                                   */
/* ----------------------------------------------------------------- */

#include "cb.h"
#include "file.h"
#include "interfc.h"
#include "memctrl.h"
#include "random.h"

#include "ctk.h"

#include <stdlib.h>    // for srand and rand
#include <time.h>      // for time
#include "math.h"
#include "parametr.c"


/* ========================== PROTOTYPES =========================== */

void PrintInitialData(int quietLevel, char *TSInput, char *TSOutput,
  int sampleSize);

void PrintInfo(void);

int main(int argc, char* argv[]);


/* ========================== FUNCTIONS ============================ */

void PrintInitialData(int quietLevel, char *TSInput, char *TSOutput,
  int sampleSize)
  {
  if (quietLevel > 1)
    {
    PrintMessage("\n%s   %s   %s\n", ProgName, VersionNumber, LastUpdated);
    PrintMessage("Input Training Set\t\t = %s\n", TSInput);
    PrintMessage("Output Training Set\t\t = %s\n", TSOutput);
    PrintMessage("Number of sample vectors\t = %d\n", sampleSize);
    PrintMessage("\n");
    }
  }  /* PrintInitialData() */

/*-------------------------------------------------------------------*/

void PrintInfo(void)
  {
  PrintMessage("\n%s   %s   %s\n", ProgName, VersionNumber, LastUpdated);
  PrintMessage("Console interface for sampling.\n");
  PrintMessage("Usage: %s [%coption] ", ProgName, OPTION_SYMBOL);
  PrintMessage("<training set> <sample of training set>\n");
  PrintMessage("For example: %s mushroom mushroom_sample\n", ProgName);
  PrintMessage("\n  Options:\n");
  PrintOptions();
  PrintMessage("\n");
  }  /* PrintInfo() */

/*-------------------------------------------------------------------*/

int main(int argc, char* argv[])
  {
  TRAININGSET   TS;
  TRAININGSET   SAMPLE;
  char          TSInput[MAXFILENAME] = {'\0'};
  char          TSOutput[MAXFILENAME] = {'\0'};
  ParameterInfo paraminfo[2] = { { TSInput,  FormatNameTS, 0, INFILE },
                                 { TSOutput, FormatNameTS, 0, OUTFILE } };
  int i;

  ParseParameters(argc, argv, 2, paraminfo);

  /* training set doesn't exist */
  if (!ExistFile(TSInput))
    {
    ErrorMessage("\nERROR: File doesn't exist: %s\n\n", TSInput);
    ExitProcessing(FATAL_ERROR);
    }

  /* result codebook exists and we are told not to overwrite */
  if (ExistFile(TSOutput) && !Value(OverWrite))
    {
    ErrorMessage("\nERROR: File already exists: %s\n\n", TSOutput);
    ExitProcessing(FATAL_ERROR);
    }

  PrintInitialData(Value(QuietLevel),
                   TSInput,
                   TSOutput,
                   Value(SampleSize)
                   );

  ReadTrainingSet(TSInput, &TS);

  if  (Value(SampleSize) > BookSize(&TS))
    {
    ErrorMessage("\nERROR: Number of vectors in training set ");
    ErrorMessage("(%d) < number of samples ", BookSize(&TS));
    ErrorMessage("(%d)!\n\n", Value(SampleSize));
    FreeCodebook(&TS);
    ExitProcessing(FATAL_ERROR);
    }

  CreateNewCodebook(&SAMPLE, Value(SampleSize),&TS);
  srand(time(0));

  for (i = 0; i < Value(SampleSize); i++)
    {
    int r = i + (rand() % (BookSize(&TS)-i));
    CopyNode(&Node(&TS, r), &Node(&SAMPLE, i), VectorSize(&SAMPLE));
    CopyNode(&Node(&TS, i), &Node(&TS, r),     VectorSize(&TS));
    }

  WriteCodebook(TSOutput, &SAMPLE, Value(OverWrite));
  FreeCodebook(&TS);
  FreeCodebook(&SAMPLE);
  checkmemory();

  return 0;
  } /* main */

