/*
$Revision: 1.3 $
$Date: 2009/12/03 13:46:01 $
$Author: apollan $
$Name:  $
$Id: catscale.c,v 1.3 2009/12/03 13:46:01 apollan Exp $
*/

/*-------------------------------------------------------------------*/
/* CATSCALE.C   Antti Pollanen                                       */
/*                                                                   */
/* Console interface for scaling.                                    */
/*                                                                   */
/*                                                                   */
/* Naming conventions used in the code                               */
/*                                                                   */
/*    TS        training set (data objects)                          */
/*    CB        codebook (cluster representatives, centroids)        */
/*    CCB       categorical data codebook                            */
/*    P         partitioning (pointing from TS to CB)                */
/*                                                                   */
/*    p-prefix  pointer, e.g. pTS is pointer to the training set TS  */
/*                                                                   */
/*-------------------------------------------------------------------*/

#define  ProgName       "catscale"
#define  VersionNumber  "Version 0.10"
#define  LastUpdated    "07.10.2009" /* AP */
#define  MAXFILENAME    64
#define  FACTFILE       "catscale.fac"

/* ----------------------------------------------------------------- */
/*      Changelog:                                                   */
/*                                                                   */
/* AP   0.10  Initial implementation                                 */
/*                                                                   */
/* ----------------------------------------------------------------- */

#include "cb.h"
#include "file.h"
#include "interfc.h"
#include "memctrl.h"
#include "random.h"

#include "ctk.h"

#include "math.h"
#include "parametr.c"


/* ========================== PROTOTYPES =========================== */

void PrintInitialData(int quietLevel, char *TSInput, char *TSOutput);

void PrintInfo(void);

int main(int argc, char* argv[]);

/* ========================== FUNCTIONS ============================ */

void PrintInitialData(int quietLevel, char *TSInput, char *TSOutput)
  {
  if (quietLevel > 1)
    {
    PrintMessage("\n%s   %s   %s\n", ProgName, VersionNumber, LastUpdated);
    PrintMessage("Input Training Set\t\t = %s\n", TSInput);
    PrintMessage("Output Training Set\t\t = %s\n", TSOutput);
    PrintMessage("\n");
    }
  }  /* PrintInitialData() */

/*-------------------------------------------------------------------*/

void PrintInfo(void)
  {
  PrintMessage("\n%s   %s   %s\n", ProgName, VersionNumber, LastUpdated);
  PrintMessage("Console interface for scaling.\n");
  PrintMessage("Usage: %s [%coption] ", ProgName, OPTION_SYMBOL);
  PrintMessage("<training set> <sample of training set>\n");
  PrintMessage("For example: %s mushroom mushroom_scaled\n", ProgName);
  PrintMessage("\n  Options:\n");
  PrintOptions();
  PrintMessage("\n");
  }  /* PrintInfo() */

/*-------------------------------------------------------------------*/

int main(int argc, char* argv[])
  {
  TRAININGSET   TS;
  char          TSInput[MAXFILENAME] = {'\0'};
  char          TSOutput[MAXFILENAME] = {'\0'};
  ParameterInfo paraminfo[2] = { { TSInput,  FormatNameTS, 0, INFILE },
                                 { TSOutput, FormatNameTS, 0, OUTFILE } };

  ParseParameters(argc, argv, 2, paraminfo);

  /* training set doesn't exist */
  if (!ExistFile(TSInput))
    {
    ErrorMessage("\nERROR: File doesn't exist: %s\n\n", TSInput);
    ExitProcessing(FATAL_ERROR);
    }

  /* result codebook exists and we are told not to overwrite */
  if (ExistFile(TSOutput) && !Value(OverWrite))
    {
    ErrorMessage("\nERROR: File already exists: %s\n\n", TSOutput);
    ExitProcessing(FATAL_ERROR);
    }

  PrintInitialData(Value(QuietLevel),
                   TSInput,
                   TSOutput);

  ReadTrainingSet(TSInput, &TS);
  CATScaleTrainingSet(&TS, &TS);
  WriteTrainingSet(TSOutput, &TS, Value(OverWrite));
  FreeCodebook(&TS);
  checkmemory();

  return 0;
  } /* main */

