/*
 $Revision: 1.22 $
 $Date: 2010/02/08 18:49:11 $
 $Author: apollan $
 $Name:  $
 $Id: ctk.h,v 1.22 2010/02/08 18:49:11 apollan Exp $
 */

#if !defined(__CTK_H)
#define __CTK_H

/* ========================== ENUMS ================================ */

typedef enum
  {
  RANDOMINIT,
  SIMPLEINIT
  } CATINITMETHOD;

typedef enum
  {
  ENTROPYDISTANCE,
  JOINTPROBABILITYDISTANCE,
  LINEARDISTANCE,
  SIMPLEMATCHINGDISTANCE
  } CATDISTANCETYPE;

typedef enum
  {
  DISTRIBUTIONREPR,
  MEDOIDREPR,
  MODEREPR,
  STMEDOIDREPR
  } CATCENTROIDTYPE;

typedef enum
  {
  ERRORMESSAGE,
  WARNINGMESSAGE,
  INFOMESSAGE,
  DEBUGMESSAGE
  } CATMESSAGETYPE;


/* ========================== DATA STRUCTS ========================= */

/* Number of instances in different categories for one attribute.    */
typedef struct
  {
  int *Instances;
  } DIVISION;

/* Divisions of attribute values for one medoid.                     */
typedef struct
  {
  DIVISION *Divisions;
  int Count;
  int *Categories;
  } CENTROIDINFO;

typedef struct
  {
  CODEBOOK *pCodebook;
  CENTROIDINFO *CentroidInformation;
  int *Categories;
  } CATCODEBOOK;

/*-------------------------------------------------------------------*/

typedef struct
  {
  int N;
  int Size;
  void** Data;
  } UTM;


/* ========================== MACROS =============================== */

#define     CategoryInstances(pCCB,prot,attr,cat) \
            pCCB->CentroidInformation[prot].Divisions[attr].Instances[cat]
#define     Categories(pCCB,attr)  pCCB->Categories[attr]
#define     Count(pCCB,prot)       pCCB->CentroidInformation[prot].Count
#define     ClusterCategories(pCCB,prot,attr) \
            pCCB->CentroidInformation[prot].Categories[attr]

#define     CATPrint(quietLevel, msgLevel, format) \
            if (msgLevel <= quietLevel) PrintMessage(format);

#define     CATPrintV(quietLevel, msgLevel, format, ...) \
            if (msgLevel <= quietLevel) PrintMessage(format, __VA_ARGS__);

// Use PrintError for real error messages


/* ========================== CONSTANTS ============================ */

#define     UNKNOWNTYPE            -1
#define     EPSILON                0.000001

/* ========================== PROTOTYPES =========================== */

 // CATCODEBOOK management
void CATScaleTrainingSet (TRAININGSET* in, TRAININGSET* out);
int  CATCreateNewCodebook(CATCODEBOOK *pCCB,
                          CODEBOOK *pCB,
                          TRAININGSET *pTS);

void CATFreeCodebook     (CATCODEBOOK* pCCB);
void CATClearDivisions   (CATCODEBOOK *pCCB);

// Clustering subroutines
double CATVectorDistance(CATCODEBOOK *pCCB,
                         PARTITIONING *pP,
                         TRAININGSET *pTS,
                         int centroid,
                         int vector,
                         CATDISTANCETYPE
                         disttype);

int CATFindNearestVector(CATCODEBOOK *pCCB,
                         PARTITIONING *pP,
                         TRAININGSET *pTS,
                         int vector,
                         CATDISTANCETYPE disttype);

void CATChangePartition (CATCODEBOOK *pCCB,
                         TRAININGSET *pTS,
                         PARTITIONING *pP,
                         int from,
                         int to, int
                         vector);

void CATMergeClusters   (CATCODEBOOK *pCCB,
                         TRAININGSET *pTS,
                         PARTITIONING *pP,
                         int cluster1,
                         int cluster);

void CATGenerateInitialSolution(
                         PARTITIONING *pP,
                         CATCODEBOOK *pCCB,
                         TRAININGSET *pTS,
                         CATINITMETHOD initMethod);

void CATGenerateOptimalCodebook(
                         PARTITIONING *pP,
                         TRAININGSET *pTS,
                         CATCODEBOOK *pCCB,
                         CATCENTROIDTYPE representativeType);

void CATGenerateMedoids( PARTITIONING *pP,
                         TRAININGSET *pTS,
                         CATCODEBOOK *pCCB,
                         UTM* distances);

int CATGenerateOptimalPartition(
                         CATCODEBOOK *pCCB,
                         TRAININGSET *pTS,
                         PARTITIONING *pP,
                         CATDISTANCETYPE disttype);

double CATClusterEntropy(CATCODEBOOK* pCCB, int i);
double CATDatasetEntropy(CATCODEBOOK* pCCB);
double CATExpectedEntropy(CATCODEBOOK* pCCB, TRAININGSET* pTS);

// Clustering methods
int PerformCKM          (PARTITIONING *pP,
                         CATCODEBOOK *pCCB,
                         TRAININGSET *pTS,
                         int kmIter,
                         int quietLevel,
                         CATDISTANCETYPE distanceMetric,
                         CATCENTROIDTYPE representativeType,
                         CATINITMETHOD initMethod);

// UTM
int   UTMIndex          (UTM* pUTM, int x, int y);
void  UTMCreate         (UTM* pUTM, int N);
void* UTMGet            (UTM* pUTM, int x, int y);
void  UTMSet            (UTM* pUTM, int x, int y, void* data);
void  UTMFree           (UTM* pUTM);

// Printing routines
void CATPrintCluster    (CATCODEBOOK *pCCB, int cluster);
void CATPrintCodebook   (CATCODEBOOK *pCCB);
void CATPrintIterationInfo
                        (int quietLevel,
                        int iteration,
                        int changes);


// Misc
void CATPermutation     (int* permutation, int size, int p);
int CATFactorial        (int i);
void InitRandom         ();

#endif /* __CTK_H */
