#!/bin/bash
# Author: Antti Pollanen
# Usage: ./run.sh clustering_properties.cp

echo ""

# Configuration from file:

if [ "$1" != "" ]; then
	. $1
else
	echo "Usage: ./run.sh clustering_properties.cp"
	echo ""
	exit
fi

echo "Settings"
echo "Method:       ${method}"
echo "Parameters:   ${parameters}"
echo "Iterations:   ${iterations}"
echo "Cluster size: ${min_clusters} - ${max_clusters}"
echo "Dataset:      ${dataset}"
echo "Real dataset: ${realdataset}"
echo "Classes:      ${classes}"
echo "Result dir:   ${resultdir}"
echo "Eval dir:     ${evaldir}"


# Remove old results.
/bin/rm -R ${resultdir}
mkdir ${resultdir}

echo ""
echo "Clustering"

C=1

echo "time = [" > ${resultdir}/timelog
echo "entropy = [" > ${resultdir}/entlog
echo "classification = [" > ${resultdir}/classlog

START=`date`

# Cluster with different cluster sizes.
for (( i = ${min_clusters}; i <= ${max_clusters}; i++ ))
do
	echo -n "."
	
	# Clustering
	for (( j = 1; j <= ${iterations}; j++ ))
	do
		/usr/bin/time -p ${method} -Cn=${i} ${parameters} ${dataset} "${resultdir}/${j}.pa" >> ${resultdir}/log 2>> ${resultdir}/temp
	done

	awk '/^user/ {print $2}' ${resultdir}/temp > ${resultdir}/time
	rm ${resultdir}/temp

	# Evaluating
	for f in `ls ${resultdir}/*.pa`
	do
		if [ ${i} = ${classes} ]
		then
			${evaldir}/cbceval -Mn=2 ${f} ${realdataset} >> ${resultdir}/classification
		fi

		${evaldir}/cbceval -Mn=0 ${f} ${dataset} >> ${resultdir}/entropy
	done

	# Statistics
	./stats.awk ${resultdir}/time >> ${resultdir}/timelog
	./stats.awk ${resultdir}/entropy >> ${resultdir}/entlog
	
	if [ ${i} = ${classes} ]
	then
		./stats.awk ${resultdir}/classification >> ${resultdir}/classlog
	fi

	# Cleanup
	rm ${resultdir}/*.pa
	rm ${resultdir}/time
	rm ${resultdir}/entropy
	
	if [ ${i} = ${classes} ]
	then
		rm ${resultdir}/classification
	fi
done

END=`date`

echo "];" >> ${resultdir}/timelog
echo "];" >> ${resultdir}/entlog
echo "];" >> ${resultdir}/classlog


# Writing final results
echo "% Clustering result"                                > ${resultdir}/final
echo "% Started:      ${START}"                          >> ${resultdir}/final
echo "% Ended:        ${END}"                            >> ${resultdir}/final
echo "% Method:       ${method}"                         >> ${resultdir}/final
echo "% Parameters:   ${parameters}"                     >> ${resultdir}/final
echo "% Iterations:   ${iterations}"                     >> ${resultdir}/final
echo "% Cluster size: ${min_clusters} - ${max_clusters}" >> ${resultdir}/final
echo "% Dataset:      ${dataset}"                        >> ${resultdir}/final
echo "% Real dataset: ${realdataset}"                    >> ${resultdir}/final
echo "% Classes:      ${classes}"                        >> ${resultdir}/final
echo "% Result dir:   ${resultdir}"                      >> ${resultdir}/final
echo "% Eval dir:     ${evaldir}"                        >> ${resultdir}/final
echo ""                                                  >> ${resultdir}/final
cat ${resultdir}/timelog                                 >> ${resultdir}/final
echo ""                                                  >> ${resultdir}/final
cat ${resultdir}/entlog                                  >> ${resultdir}/final
echo ""                                                  >> ${resultdir}/final
cat ${resultdir}/classlog                                >> ${resultdir}/final


echo " "
echo ""
echo "Completed"
echo ""
